package org.example.auth

import io.ktor.server.routing.Route
import io.ktor.server.routing.route
import org.example.data.DataRepository
import io.ktor.server.application.*
import io.ktor.server.auth.authenticate
import io.ktor.server.auth.jwt.JWTPrincipal
import io.ktor.server.auth.principal
import io.ktor.server.request.*
import io.ktor.server.response.*
import io.ktor.server.routing.*
import org.example.dto.requests.AddToCartRequest
import org.example.dto.responses.CartResponse
import org.example.dto.responses.FavoritesResponse
import org.example.dto.responses.OperationResponse
import org.example.dto.responses.SneakerResponse

fun Route.sneakerRoutes() {
    route("/sneakers") {
        get {
            val sneakers = DataRepository.getAllSneakers()
            call.respond(SneakerResponse(sneakers))
        }

        authenticate {
            route("/favorites") {
                get {
                    val userId = call.principal<JWTPrincipal>()?.payload?.getClaim("userId")?.asInt()
                        ?: throw IllegalArgumentException("Invalid user")

                    val favoriteIds = DataRepository.getFavorites(userId)
                    val sneakers = favoriteIds.mapNotNull { DataRepository.getSneakerById(it) }

                    call.respond(FavoritesResponse(favoriteIds, sneakers))
                }

                post {
                    val userId = call.principal<JWTPrincipal>()?.payload?.getClaim("userId")?.asInt()
                        ?: throw IllegalArgumentException("Invalid user")
                    val sneakerId = call.request.queryParameters["sneakerId"]?.toIntOrNull()
                        ?: throw IllegalArgumentException("Missing sneakerId")

                    val success = DataRepository.addToFavorites(userId, sneakerId)
                    call.respond(
                        OperationResponse(
                            success,
                            if (success) "Added to favorites" else "Failed to add to favorites"
                        )
                    )
                }

                delete {
                    val userId = call.principal<JWTPrincipal>()?.payload?.getClaim("userId")?.asInt()
                        ?: throw IllegalArgumentException("Invalid user")
                    val sneakerId = call.request.queryParameters["sneakerId"]?.toIntOrNull()
                        ?: throw IllegalArgumentException("Missing sneakerId")

                    val success = DataRepository.removeFromFavorites(userId, sneakerId)
                    call.respond(OperationResponse(success, if (success) "Removed from favorites" else "Failed to remove from favorites"))
                }
            }

            route("/cart") {
                get {
                    val userId = call.principal<JWTPrincipal>()?.payload?.getClaim("userId")?.asInt()
                        ?: throw IllegalArgumentException("Invalid user")

                    val cartItems = DataRepository.getCart(userId)
                    val sneakers = cartItems.mapNotNull { DataRepository.getSneakerById(it.sneakerId) }

                    call.respond(CartResponse(cartItems, sneakers))
                }

                post {
                    val userId = call.principal<JWTPrincipal>()?.payload?.getClaim("userId")?.asInt()
                        ?: throw IllegalArgumentException("Invalid user")
                    val request = call.receive<AddToCartRequest>()

                    val success = DataRepository.addToCart(userId, request.sneakerId, request.quantity)
                    call.respond(OperationResponse(success, if (success) "Added to cart" else "Failed to add to cart"))
                }

                delete {
                    val userId = call.principal<JWTPrincipal>()?.payload?.getClaim("userId")?.asInt()
                        ?: throw IllegalArgumentException("Invalid user")
                    val sneakerId = call.request.queryParameters["sneakerId"]?.toIntOrNull()
                        ?: throw IllegalArgumentException("Missing sneakerId")

                    val success = DataRepository.removeFromCart(userId, sneakerId)
                    call.respond(OperationResponse(success, if (success) "Removed from cart" else "Failed to remove from cart"))
                }
            }
        }
    }
}